﻿using System;
using System.Windows.Forms;
using ChatServer;
using RemObjects.SDK;

namespace ChatClient
{
	public partial class MainForm : Form, IChatEvents
	{
		private delegate void GuiUpdate();

		void IChatEvents.MessageReceived(string sender, string message)
		{
			this.Invoke(new GuiUpdate(() => this.ChatLog.Items.Add(string.Format(@"{0}: {1}", sender, message))));
		}

		private const string SERVER_URL = @"http://localhost:8099/bin";

		private IClientChannel _clientChannel;
		private IMessage _message;
		private ILoginService_Async _loginServiceAsync;
		private IChatService_Async _chatServiceAsync;
		private EventReceiver _eventReceiver;

		public MainForm()
		{
			InitializeComponent();
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			this._clientChannel = ClientChannel.ChannelMatchingTargetUri(SERVER_URL);
			this._message = RemObjects.SDK.Message.MessageMatchingTargetUri(SERVER_URL);

			this._loginServiceAsync = new LoginService_AsyncProxy(this._message, this._clientChannel);
			this._chatServiceAsync = new ChatService_AsyncProxy(this._message, this._clientChannel);

			this._eventReceiver = new EventReceiver();
			this._eventReceiver.Channel = this._clientChannel;
			this._eventReceiver.Message = this._message;
		}

		private async void LoginButton_Click(object sender, EventArgs e)
		{
			string username = this.UserNameTextBox.Text;

			if (string.IsNullOrWhiteSpace(username))
			{
				return;
			}

			username = username.Replace("\"", "\"\"");

			this.Enabled = false;

			try
			{
				string loginString = string.Format("User Id=\"{0}\";Password=\"{1}\"", username, username);
				bool loginResult = await this._loginServiceAsync.LogInAsync(loginString);

				if (loginResult)
				{
					this._eventReceiver.RegisterEventHandler(this);
				}

				this.UserNameTextBox.Enabled = !loginResult;
				this.LoginButton.Enabled = !loginResult;
				this.LogoutButton.Enabled = loginResult;
				this.SendButton.Enabled = loginResult;
			}
			finally
			{
				this.Enabled = true;
			}

		}

		private async void LogoutButton_Click(object sender, EventArgs e)
		{
			this.Enabled = false;

			try
			{
				this._eventReceiver.UnregisterEventHandler(this);
				await this._loginServiceAsync.LogOutAsync();

				this.UserNameTextBox.Enabled = true;
				this.LoginButton.Enabled = true;
				this.LogoutButton.Enabled = false;
				this.SendButton.Enabled = false;
			}
			finally
			{
				this.Enabled = true;
			}
		}

		private async void SendButton_Click(object sender, EventArgs e)
		{
			string message = this.MessageTextBox.Text;

			if (string.IsNullOrEmpty(message))
			{
				return;
			}

			this.Enabled = false;
			try
			{
				await this._chatServiceAsync.SendMessageAsync(message);
			}
			finally
			{
				this.Enabled = true;
			}
		}
	}
}
